/**
 * @doc
 * @anue-package Anue.API.Profile
 * 使用者帳戶資訊相關的API實作與集合，使用方式
 *
 * ```Javascript
 * import 'anue-fe-sdk'
 * import 'anue-fe-sdk/web-preset'
 * import Profile from 'anue-fe-sdk/API/Profile'
 *
 * // ...
 *
 * ```
 */

import * as Network from '@libraries/Network';
import Endpoints from '@constants/Endpoints';
import Misc from '@constants/Misc';
import API from '@packages/APIs/API';

type UpdatePassword = APIResponse.Response<
  APIResponse.v1.User.Profile.Password
>;

type RecoverPassword = APIResponse.Response<
  APIResponse.v1.User.Profile.Password.Recovery
>;

type ResetPassword = APIResponse.Response<
  APIResponse.v1.User.Profile.Password.Reset
>;

export default {

  updateProfileV1: (profile: Anue.Auth.UserProfile) => {

    return new API().options({
      method: Misc.RequestMethod.PUT,
      url: Endpoints.v1User.profile(),
      body: profile
    })
    .transform(res => res.statusCode < 300)
  },

  // @TODO 確認token是否會跟著Network送的request一起出去
  updatePassword: async (
    oldPassword: string,
    newPassword: string
  ): Promise<number> => {
    const resp = await Network.getDriver().send<UpdatePassword>({
      method: Misc.RequestMethod.PUT,
      url: Endpoints.v1User.profile.password(),
      body: {
        oldPassword,
        newPassword
      }
    });
    const code = await resp.statusCode;

    if (code !== 200) {
      throw {
        message: 'failed to update password',
        detail: resp
      };
    }
    return code;
  },

  recoverPassword: async (email: string): Promise<number> => {
    const resp = await Network.getDriver().send<RecoverPassword>({
      method: Misc.RequestMethod.POST,
      url: Endpoints.v1User.profile.password.recovery(),
      body: { email }
    });
    const code = await resp.statusCode;
    if (code !== 200) {
      throw {
        message: 'failed to send recovery email',
        detail: resp
      };
    }
    return code;
  },

  resetPassword: async (
    recoverKey: string,
    password: string
  ): Promise<number> => {
    const resp = await Network.getDriver().send<ResetPassword>({
      method: Misc.RequestMethod.PUT,
      url: Endpoints.v1User.profile.password.reset(),
      body: {
        recoverKey,
        password
      }
    });
    const code = await resp.statusCode;
    if (code !== 200) {
      throw {
        message: 'failed to confirm password reset',
        detail: resp
      };
    }
    return code;
  },

  getProfileV2: (): API<Anue.Auth.UserProfile> => {
    return new API<Anue.Auth.UserProfile>().options({
      url: Endpoints.api.v2.user.new.profile()
    })
  }

};
