import Eventy from '@utils/eventy';
import Qable from '@utils/qable';
import Endpoints from '@constants/Endpoints';
import * as Network from '@libraries/Network';

export type ProviderResponse = Anue.Provider.NullableResponse;
export type ChangeHandler = Anue.Provider.ChangeHandler;

export default abstract class AbstractProvider {
  protected mContext: Anue.Provider.Response | null = null;
  private mEvent = new Eventy();
  private mInitialized = false;

  abstract loginImpl(args?: any): Promise<ProviderResponse | null>;
  abstract logoutImpl(): Promise<boolean>;
  abstract getName(): string;
  abstract refresh(): Promise<ProviderResponse>;
  abstract initImpl(): Promise<AbstractProvider>;
  abstract getContext(): ProviderResponse | null;

  mode = 0;

  protected emitStateChange = () => {
    this.mEvent.emit('login', this.mContext);
  };

  renewer: (() => Promise<Anue.Auth.RefreshTokenResponse>) | null = null;

  getProfile = async (args: Anue.Auth.GetProfileArgs) => {
    const resp = await Network.getDriver().send<
      APIResponse.Response<APIResponse.v2.User.Profile>
    >({
      url: Endpoints.api.v2.user.profile(),
      headers: {
        authorization: args.token,
        'X-CNYES-APP': args.channel
      }
    });
    if (resp.statusCode !== 200) {
      return null;
    }
    return await {
      name: resp.items.name,
      email: resp.items.email,
      uid: resp.items.uid,
      avatar: resp.items.avatar
    };
  };

  @Qable
  init = async (): Promise<AbstractProvider> => {
    if (this.mInitialized) {
      return await this;
    }
    console.log(`[${this.getName()}] initializing ⏱`);
    await this.initImpl();
    this.mInitialized = true;
    console.log(`[${this.getName()}] initialize 👍`);
    return await this;
  };

  isLoggedIn = (): boolean => {
    return this.mContext !== null;
  };

  login = async (args?: any): Promise<ProviderResponse | null> => {
    await this.loginImpl(args);
    this.emitStateChange()
    console.log('Provider [%s] logged in ', this.getName());
    return this.mContext;
  };

  logout = async (): Promise<boolean> => {
    const success = await this.logoutImpl();
    this.mContext = null;
    console.log('Provider [%s] logged out ', this.getName());
    return success;
  };

  onLoginStateChange = (fn: ChangeHandler): AbstractProvider => {
    this.mEvent.on('login', fn);
    return this;
  };
}
