import AbstractProvider from '@packages/Provider/AbstractProvider.abstract';
import Eventy from '@utils/eventy'

export default class AnueProxyProvider extends AbstractProvider {

  private event = new Eventy()

  loginImpl = async (args?: any): Promise<Anue.Provider.NullableResponse> => {
    return {
      idToken: '',
      accessToken: 'proxy',
      tokenType: ''
    }
  }

  logoutImpl = async (): Promise<boolean> => {
    return true
  }

  getName(): string {
    return 'proxy'
  }

  renewer = async (): Promise<Anue.Auth.RefreshTokenResponse> => {

    window.postMessage(JSON.stringify({
      source: 'proxy',
      action: 'refreshToken'
    }), '*')

    return new Promise<Anue.Auth.RefreshTokenResponse>(resolve => {

      const remove = this.event.on('refresh', (data: Anue.Auth.RefreshTokenResponse) => {
        resolve(data)
        remove()
      })

    })

  }

  refresh(): Promise<Anue.Provider.NullableResponse> {
    throw new Error('Method not implemented.');
  }

  initImpl = async (): Promise<AbstractProvider> => {

    window.postMessage(JSON.stringify({
      source: 'proxy',
      action: 'refreshToken'
    }), '*')

    this.mContext = {
      idToken: '',
      accessToken: 'proxy',
      tokenType: ''
    }

    document.addEventListener('message', (e: any) => {
      try {
        const payload = JSON.parse(e.data)
        if (payload.type !== 'anue-proxy') {
          return
        }
        if (payload.data) {
          this.event.emit('refresh', {
            statusCode: 200,
            message: '',
            items: payload.data
          })
          this.emitStateChange()
        }
      }
      catch (err) {
        console.log(err)
      }
    })

    return this
  }

  getContext(): Anue.Provider.NullableResponse {
    return this.mContext
  }

}
