import { Web } from '@utils/isomorphy';
import Eventy from '@utils/eventy';

const event: Eventy = new Eventy();

if (typeof window !== undefined) {
  window.addEventListener('message', e => {
    if (e.origin.includes('cnyes.com') || e.origin.includes('cnyes.cool')) {
      try {
        const data = JSON.parse(e.data);
        if (data.type) {
          const eventToken = `dpi:${data.tid}`;
          console.log(
            '%c [gauth-rpc] received message token = %s %o',
            'color: green',
            eventToken,
            data
          );
          event.emit(eventToken, data);
        }
      } catch (ignored) {}
    }
  });
}

export default class RPC {
  private mProxyUri: string = '';
  private mDpi: HTMLIFrameElement;
  private mTag: string;
  private mTid: number = 1;

  constructor(tag: string, proxyUri: string) {
    if (!tag || !proxyUri) {
      throw Error(
        `[rpc] can not be initialized with tag=${tag} proxyUri=${proxyUri}`
      );
    }

    this.mProxyUri = proxyUri;
    this.mTag = tag;

    const frameId = `anue-dpi-${this.mTag}`;

    this.mDpi = document.getElementById(frameId) as HTMLIFrameElement;
    if (!this.mDpi) {
      this.mDpi = document.createElement('iframe');
      this.mDpi.setAttribute('id', frameId);
      this.mDpi.style.border = 'none';
      this.mDpi.style.position = 'fixed';
      this.mDpi.style.left = '-9999px';
      this.mDpi.style.top = '-9999px';
      this.mDpi.height = '1px';
      this.mDpi.width = '1px';
      document.body.appendChild(this.mDpi);
    }
  }

  @Web(null)
  rpcCall = async (prefix, ...args): Promise<Anue.DPI.Payload> => {
    return new Promise<Anue.DPI.Payload>((resolve, reject) => {
      try {
        // TODO: better way to detect rpc fail
        const tm = setTimeout(() => {
          reject('[rpc] failed to load');
          clearTimeout(tm);
        }, 10000);
        const eventToken = `dpi:${this.mTag}.${++this.mTid}`;
        const revoke = event.on<Anue.DPI.Payload>(
          eventToken,
          (data: Anue.DPI.Payload) => {
            clearTimeout(tm);
            if (!data.error) {
              resolve(data);
            } else {
              reject(data);
            }
            revoke();
          }
        );
        console.log(
          '%c[rpc:%s] event is deferring [%s]',
          'color: brown',
          this.mTag,
          eventToken
        );
        console.log('[rpc:%s] call %s %o', this.mTag, prefix, args);
        this.mDpi.src = `${this.mProxyUri}?dpi=${prefix}:${encodeURIComponent(
          args.join(' ')
        )}&state=${this.mTag}.${this.mTid}`;
      } catch (err) {
        reject(err);
      }
    });
  };
}
