const path = require('path')
const bundleAnalyzer = require('webpack-bundle-analyzer').BundleAnalyzerPlugin
const TsconfigPathsPlugin = require('tsconfig-paths-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin')
const fs = require('fs')
const outputProject = process.env.external_output

const utils = fs.readdirSync(path.resolve(__dirname, './src/utils')).reduce((p, f) => {
  if (!f.startsWith('__') && !f.startsWith('index.')) {
    p[f.replace('.ts', '')] = `./src/utils/${f}`
  }
  return p
}, {})

console.log('import utils', utils)

module.exports = {
  mode: 'development',
  entry: {
    ...utils,
    'old-driver-preset': './src/presets/old-driver/index.ts',
    'preset-init': './src/presets/preset-init.ts',
    'web-preset': './src/presets/web/index.ts',
    'react-native-preset': './src/presets/react-native/index.ts',
    Auth: './src/packages/Auth/index.ts',
    APIs: './src/packages/APIs/index.ts',
    Network: './src/libraries/Network/index.ts',
    Storage: './src/libraries/Storage/index.ts',
    APIs: './src/packages/APIs/index.ts',
    Constants: './src/constants/index.ts',
    FacebookWebProvider: './src/packages/Provider/FacebookWebProvider.ts',
    GoogleWebProvider: './src/packages/Provider/GoogleWebProvider.ts',
    FacebookOAuth2Provider: './src/packages/Provider/FacebookOAuth2Provider.ts',
    GoogleOAuth2Provider: './src/packages/Provider/GoogleOAuth2Provider.ts',
    AnueMailProvider: './src/packages/Provider/AnueMailProvider.ts',
    AnueProxyProvider: './src/packages/Provider/AnueProxyProvider.ts'
  },
  externals: {
    axios: 'axios',
  },
  module: {
    rules: [{
      test: /\.ts$/,
      use: 'ts-loader',
      exclude: /node_modules/
    }]
  },
  devtool: 'source-map',
  optimization: {
    runtimeChunk: {
      name: 'runtime'
    },
    splitChunks: {
      cacheGroups: {
        common: {
          name: 'common',
          chunks: 'all',
          minChunks: 2,
          enforce: true
        }
      }
    }
  },
  resolve: {
    plugins: [
      new TsconfigPathsPlugin({
        configFile: './tsconfig.json',
        baseUrl: path.resolve(__dirname)
      })
    ],
    alias: {
      "@constants": path.resolve(__dirname, "./src/constants"),
      "@mocks": path.resolve(__dirname, "./src/mocks"),
      "@utils": path.resolve(__dirname, "./src/utils"),
      "@defines": path.resolve(__dirname, "./src/defines"),
      "@libraries": path.resolve(__dirname, "./src/libraries"),
      "@packages": path.resolve(__dirname, "./src/packages"),
    },
    extensions: ['.ts', '.js'],
  },
  output: {
    filename: '[name].js',
    library: ['anue-fe-sdk', '[name]'],
    path: outputProject ? outputProject : path.resolve(__dirname, 'build'),
    libraryTarget: "umd",
    umdNamedDefine: true,
    globalObject: '(typeof window === \'undefined\' ? global : window)',
  },
  plugins: [
    new CopyWebpackPlugin([{
      from: path.resolve('./bundle-remap.js'),
      to: outputProject
        ? path.resolve(outputProject, './index.js')
        : path.resolve('./build/index.js')
    }, {
      from: './src/defines',
      to: outputProject
      ? path.resolve(outputProject, './defines/')
      : path.resolve('./build/defines/')
    }])
  ]
  .concat(process.env.ANALYZE ? [
    new bundleAnalyzer()
  ]: [])

}